@@ -10,11 +10,11 @@ training patterns: trainer.fit(model, datamodule).
 from abc import abstractmethod
 from typing import Optional, Union, Tuple, Any, Dict, TYPE_CHECKING
 from pathlib import Path
+from datetime import datetime
 import numpy as np
+import torch
 import pytorch_lightning as pl
-
-if TYPE_CHECKING:
-    from ..nowcast import Nowcast
+import pandas as pd
 
 from ..config import DFMConfig, make_config_source, ConfigSource, MergedConfigSource
 from ..config.results import BaseResult
@@ -24,48 +24,6 @@ from ..logger import get_logger
 _logger = get_logger(__name__)
 
 
-def format_error_message(
-    model_type: str,
-    operation: str,
-    reason: str,
-    guidance: Optional[str] = None
-) -> str:
-    """Format standardized error message (standalone utility function).
-    
-    This is a standalone utility function that can be used by any class,
-    including low-level classes that don't inherit from BaseFactorModel.
-    It provides a consistent error message format across all models.
-    
-    Format: "{ModelType} {operation} failed: {reason}. {guidance}"
-    
-    Parameters
-    ----------
-    model_type : str
-        Name of the model class (e.g., "DDFM", "DDFMModel", "DFM")
-    operation : str
-        Name of the operation that failed (e.g., "prediction", "config loading")
-    reason : str
-        Reason for the failure (e.g., "model has not been trained yet")
-    guidance : str, optional
-        Actionable guidance for the user (e.g., "Please call trainer.fit(model, data_module) first")
-        
-    Returns
-    -------
-    str
-        Formatted error message
-        
-    Examples
-    --------
-    >>> format_error_message("DDFM", "prediction", "model has not been trained yet",
-    ...                      "Please call trainer.fit(model, data_module) first")
-    'DDFM prediction failed: model has not been trained yet. Please call trainer.fit(model, data_module) first'
-    """
-    message = f"{model_type} {operation} failed: {reason}."
-    if guidance:
-        message += f" {guidance}"
-    return message
-
-
 class BaseFactorModel(pl.LightningModule):
     """Base class for all factor models (PyTorch Lightning module).
     
@@ -109,70 +67,6 @@ class BaseFactorModel(pl.LightningModule):
             )
         return self._config
     
-    def _format_error_message(
-        self,
-        operation: str,
-        reason: str,
-        guidance: Optional[str] = None
-    ) -> str:
-        """Format standardized error message.
-        
-        This helper method provides a consistent error message format across all models.
-        It uses the standalone format_error_message() utility function for consistency.
-        Format: "{ModelType} {operation} failed: {reason}. {guidance}"
-        
-        Parameters
-        ----------
-        operation : str
-            Name of the operation that failed (e.g., "prediction", "config loading")
-        reason : str
-            Reason for the failure (e.g., "model has not been trained yet")
-        guidance : str, optional
-            Actionable guidance for the user (e.g., "Please call trainer.fit(model, data_module) first")
-            
-        Returns
-        -------
-        str
-            Formatted error message
-        """
-        model_type = self.__class__.__name__
-        return format_error_message(model_type, operation, reason, guidance)
-    
-    def _format_warning_message(
-        self,
-        operation: str,
-        issue: str,
-        context: Optional[str] = None,
-        suggestion: Optional[str] = None
-    ) -> str:
-        """Format standardized warning message.
-        
-        This helper method provides a consistent warning message format across all models.
-        Format: "{ModelType} {operation}: {issue}. {context} {suggestion}"
-        
-        Parameters
-        ----------
-        operation : str
-            Name of the operation (e.g., "factor validation", "training", "prediction")
-        issue : str
-            Description of the issue (e.g., "3/5 factors are constant")
-        context : str, optional
-            Additional context (e.g., "Constant factor indices: [0, 2]")
-        suggestion : str, optional
-            Actionable suggestion for the user (e.g., "This may indicate training issues")
-            
-        Returns
-        -------
-        str
-            Formatted warning message
-        """
-        model_type = self.__class__.__name__
-        message = f"{model_type} {operation}: {issue}"
-        if context:
-            message += f". {context}"
-        if suggestion:
-            message += f". {suggestion}"
-        return message
     
     def _check_trained(self) -> None:
         """Check if model is trained, raise error if not.
@@ -192,12 +86,10 @@ class BaseFactorModel(pl.LightningModule):
                     # get_result() not implemented or failed, model not fully trained
                     pass
             
-            error_msg = self._format_error_message(
-                operation="operation",
-                reason="model has not been trained yet",
-                guidance="Please call trainer.fit(model, data_module) first"
+            raise ValueError(
+                f"{self.__class__.__name__} operation failed: model has not been trained yet. "
+                f"Please call trainer.fit(model, data_module) first"
             )
-            raise ValueError(error_msg)
     
     def _check_finite(
         self, 
@@ -229,23 +121,19 @@ class BaseFactorModel(pl.LightningModule):
         if not np.all(np.isfinite(arr)):
             nan_count = np.sum(~np.isfinite(arr))
             context_str = f" {context}" if context else ""
-            warning_msg = self._format_warning_message(
-                operation="numerical stability check",
-                issue=f"{name} contains {nan_count} NaN/Inf values{context_str}",
-                context=f"Shape: {arr.shape}"
+            _logger.warning(
+                f"{self.__class__.__name__} numerical stability check: {name} contains {nan_count} NaN/Inf values{context_str}. "
+                f"Shape: {arr.shape}"
             )
-            _logger.warning(warning_msg)
             if fallback is not None:
                 _logger.info(f"{self.__class__.__name__}: Using fallback for {name}")
                 return fallback
             else:
                 # Replace NaN/Inf with finite values as last resort
                 arr_clean = np.nan_to_num(arr, nan=0.0, posinf=1e6, neginf=-1e6)
-                warning_msg = self._format_warning_message(
-                    operation="numerical stability check",
-                    issue=f"replaced NaN/Inf in {name} with finite values"
+                _logger.warning(
+                    f"{self.__class__.__name__} numerical stability check: replaced NaN/Inf in {name} with finite values"
                 )
-                _logger.warning(warning_msg)
                 return arr_clean
         return arr
     
@@ -300,54 +188,353 @@ class BaseFactorModel(pl.LightningModule):
         
         return config
     
-    def _get_data_from_datamodule(self) -> Tuple[Any, Optional[np.ndarray], Optional[np.ndarray]]:
-        """Get processed data and standardization parameters from DataModule.
-        
-        This helper method consolidates the common pattern of retrieving processed data
-        and standardization parameters (Mx, Wx) from the DataModule. It's used by both
-        DFM and DDFM in their on_train_start() methods.
+    def _get_datamodule(self):
+        """Get DataModule from model or trainer.
         
         Returns
         -------
-        X_torch : torch.Tensor
-            Processed data tensor (T x N)
-        Mx : np.ndarray or None
-            Mean values for unstandardization (N,)
-        Wx : np.ndarray or None
-            Standard deviation values for unstandardization (N,)
+        DataModule
+            DataModule instance
             
         Raises
         ------
-        RuntimeError
-            If DataModule is not available or setup() has not been called
+        ValueError
+            If DataModule is not available
         """
-        if self._data_module is None:
-            # Try to get from trainer if available
-            if hasattr(self, 'trainer') and self.trainer is not None:
-                if hasattr(self.trainer, 'datamodule') and self.trainer.datamodule is not None:
-                    self._data_module = self.trainer.datamodule
-                else:
-                    error_msg = self._format_error_message(
-                        operation="data retrieval",
-                        reason="DataModule is not available",
-                        guidance="Please ensure trainer.fit(model, data_module) is called with a DataModule"
-                    )
-                    raise RuntimeError(error_msg)
+        data_module = self._data_module
+        if data_module is None and hasattr(self, 'trainer') and self.trainer is not None:
+            data_module = getattr(self.trainer, 'datamodule', None)
+        
+        if data_module is None:
+            raise ValueError(
+                f"{self.__class__.__name__}: DataModule not available. "
+                f"Please ensure DataModule is attached to model or trainer"
+            )
+        return data_module
+    
+    def _forecast_var_factors(
+        self,
+        Z_last: np.ndarray,
+        A: np.ndarray,
+        p: int,
+        horizon: int,
+        Z_prev: Optional[np.ndarray] = None
+    ) -> np.ndarray:
+        """Forecast factors using VAR dynamics.
+        
+        Supports VAR(1) and VAR(2) factor dynamics (maximum supported order is VAR(2)).
+        
+        Parameters
+        ----------
+        Z_last : np.ndarray
+            Last factor state (m,)
+        A : np.ndarray
+            Transition matrix. For VAR(1): (m x m), for VAR(2): (m x 2m)
+        p : int
+            VAR order. Must be 1 or 2 (maximum supported order is VAR(2))
+        horizon : int
+            Number of periods to forecast
+        Z_prev : np.ndarray, optional
+            Previous factor state for VAR(2) (m,). Required if p == 2.
+            
+        Returns
+        -------
+        np.ndarray
+            Forecasted factors (horizon x m)
+        """
+        if p == 1:
+            # VAR(1): f_t = A @ f_{t-1}
+            Z_forecast = np.zeros((horizon, Z_last.shape[0]))
+            Z_forecast[0, :] = A @ Z_last
+            for h in range(1, horizon):
+                Z_forecast[h, :] = A @ Z_forecast[h - 1, :]
+        elif p == 2:
+            # VAR(2): f_t = A1 @ f_{t-1} + A2 @ f_{t-2}
+            if Z_prev is None:
+                # Fallback to VAR(1) if not enough history
+                A1 = A[:, :Z_last.shape[0]]
+                Z_forecast = np.zeros((horizon, Z_last.shape[0]))
+                Z_forecast[0, :] = A1 @ Z_last
+                for h in range(1, horizon):
+                    Z_forecast[h, :] = A1 @ Z_forecast[h - 1, :]
             else:
-                error_msg = self._format_error_message(
-                    operation="data retrieval",
-                    reason="DataModule is not available and trainer is not attached",
-                    guidance="Please ensure trainer.fit(model, data_module) is called with a DataModule"
+                A1 = A[:, :Z_last.shape[0]]
+                A2 = A[:, Z_last.shape[0]:]
+                Z_forecast = np.zeros((horizon, Z_last.shape[0]))
+                Z_forecast[0, :] = A1 @ Z_last + A2 @ Z_prev
+                if horizon > 1:
+                    Z_forecast[1, :] = A1 @ Z_forecast[0, :] + A2 @ Z_last
+                for h in range(2, horizon):
+                    Z_forecast[h, :] = A1 @ Z_forecast[h - 1, :] + A2 @ Z_forecast[h - 2, :]
+        else:
+            raise ValueError(
+                f"{self.__class__.__name__} prediction failed: unsupported VAR order {p}. "
+                f"Maximum supported VAR order is VAR(2). Please use factor_order=1 (VAR(1)) or factor_order=2 (VAR(2))"
+            )
+        return Z_forecast
+    
+    def _transform_factors_to_observations(
+        self,
+        Z_forecast: np.ndarray,
+        C: np.ndarray,
+        Wx: np.ndarray,
+        Mx: np.ndarray
+    ) -> np.ndarray:
+        """Transform forecasted factors to observed series.
+        
+        Parameters
+        ----------
+        Z_forecast : np.ndarray
+            Forecasted factors (horizon x m)
+        C : np.ndarray
+            Loading matrix (N x m)
+        Wx : np.ndarray
+            Standard deviation values for unstandardization (N,)
+        Mx : np.ndarray
+            Mean values for unstandardization (N,)
+            
+        Returns
+        -------
+        np.ndarray
+            Forecasted observations (horizon x N)
+        """
+        X_forecast_std = Z_forecast @ C.T
+        X_forecast = X_forecast_std * Wx + Mx
+        return X_forecast
+    
+    def _standardize_data(self, X: np.ndarray, Mx: Optional[np.ndarray], Wx: Optional[np.ndarray]) -> np.ndarray:
+        """Standardize data using Mx and Wx.
+        
+        Parameters
+        ----------
+        X : np.ndarray
+            Data to standardize (T x N)
+        Mx : np.ndarray or None
+            Mean values (N,)
+        Wx : np.ndarray or None
+            Standard deviation values (N,)
+            
+        Returns
+        -------
+        np.ndarray
+            Standardized data (T x N)
+        """
+        if Mx is None or Wx is None:
+            return X
+        Wx_safe = np.where(Wx == 0, 1.0, Wx)
+        return (X - Mx) / Wx_safe
+    
+    def _update_factor_state_with_history(
+        self,
+        history: int,
+        result: 'BaseResult',
+        kalman_filter: Optional[Any] = None
+    ) -> Optional[np.ndarray]:
+        """Update factor state using recent N periods of data.
+        
+        Parameters
+        ----------
+        history : int
+            Number of recent periods to use for state update
+        result : BaseResult
+            Model result containing parameters
+        kalman_filter : Any, optional
+            Kalman filter instance
+            
+        Returns
+        -------
+        np.ndarray or None
+            Updated last factor state (m,), or None if update failed
+        """
+        try:
+            data_module = self._get_datamodule()
+            X_torch = data_module.get_processed_data()
+            X_data = X_torch.cpu().numpy() if isinstance(X_torch, torch.Tensor) else np.asarray(X_torch)
+            
+            # Slice to recent N periods
+            X_recent = X_data[-history:, :] if X_data.shape[0] > history else X_data
+            
+            # Standardize and handle NaN
+            X_recent_std = self._standardize_data(X_recent, result.Mx, result.Wx)
+            X_recent_std = np.where(np.isfinite(X_recent_std), X_recent_std, np.nan)
+            
+            # Model-specific state update
+            if hasattr(self, 'encoder') and self.encoder is not None:
+                return self._update_factor_state_ddfm(X_recent_std, result, kalman_filter)
+            else:
+                return self._update_factor_state_dfm(X_recent_std, result, kalman_filter)
+        except Exception as e:
+            _logger.warning(
+                f"{self.__class__.__name__} predict(): Failed to update factor state with history={history}, "
+                f"using training state instead. Error: {type(e).__name__}: {str(e)}"
+            )
+            return None
+    
+    def _get_kalman_filter(self, kalman_filter: Optional[Any] = None):
+        """Get or create Kalman filter instance."""
+        if kalman_filter is not None:
+            return kalman_filter
+        if hasattr(self, 'kalman') and self.kalman is not None:
+            return self.kalman
+        from ..ssm.kalman import KalmanFilter
+        return KalmanFilter(
+            min_eigenval=1e-8,
+            inv_regularization=1e-6,
+            cholesky_regularization=1e-8
+        )
+    
+    def _result_to_torch_params(self, result: 'BaseResult', C: Optional[np.ndarray] = None, R: Optional[np.ndarray] = None) -> Dict[str, torch.Tensor]:
+        """Convert result parameters to torch tensors.
+        
+        Parameters
+        ----------
+        result : BaseResult
+            Model result containing parameters
+        C : np.ndarray, optional
+            Override C matrix (for DDFM where C=I)
+        R : np.ndarray, optional
+            Override R matrix (for DDFM where R=small noise)
+            
+        Returns
+        -------
+        Dict[str, torch.Tensor]
+            Dictionary of torch tensors: A, C, Q, R, Z_0, V_0
+        """
+        # Ensure V_0 is always 2D (defensive check for edge cases)
+        V_0 = result.V_0
+        if isinstance(V_0, np.ndarray):
+            if V_0.ndim == 0:
+                V_0 = np.atleast_2d(V_0)
+            elif V_0.ndim == 1:
+                # If 1D, reshape to (m x m) - assume it's a single value for m=1 case
+                V_0 = np.atleast_2d(V_0).T if V_0.shape[0] == 1 else np.atleast_2d(V_0)
+        
+        return {
+            'A': torch.tensor(result.A, dtype=torch.float32),
+            'C': torch.tensor(C if C is not None else result.C, dtype=torch.float32),
+            'Q': torch.tensor(result.Q, dtype=torch.float32),
+            'R': torch.tensor(R if R is not None else result.R, dtype=torch.float32),
+            'Z_0': torch.tensor(result.Z_0, dtype=torch.float32),
+            'V_0': torch.tensor(V_0, dtype=torch.float32)
+        }
+    
+    def _update_factor_state_dfm(
+        self,
+        X_recent_std: np.ndarray,
+        result: 'BaseResult',
+        kalman_filter: Optional[Any] = None
+    ) -> np.ndarray:
+        """Update factor state for DFM using Kalman filter.
+        
+        Parameters
+        ----------
+        X_recent_std : np.ndarray
+            Standardized recent data (T x N)
+        result : BaseResult
+            Model result containing parameters
+        kalman_filter : Any, optional
+            Kalman filter instance
+            
+        Returns
+        -------
+        np.ndarray
+            Updated last factor state (m,)
+        """
+        # Convert to torch: (N x T) format for Kalman filter
+        Y = torch.tensor(X_recent_std.T, dtype=torch.float32)
+        params = self._result_to_torch_params(result)
+        
+        # Run Kalman smoother
+        kf = self._get_kalman_filter(kalman_filter)
+        zsmooth, _, _, _ = kf(Y, params['A'], params['C'], params['Q'],
+                              params['R'], params['Z_0'], params['V_0'])
+        
+        return zsmooth.T[-1, :].cpu().numpy()
+    
+    def _update_factor_state_ddfm(
+        self,
+        X_recent_std: np.ndarray,
+        result: 'BaseResult',
+        kalman_filter: Optional[Any] = None
+    ) -> Optional[np.ndarray]:
+        """Update factor state for DDFM using encoder and Kalman filter.
+        
+        Parameters
+        ----------
+        X_recent_std : np.ndarray
+            Standardized recent data (T x N)
+        result : BaseResult
+            Model result containing parameters
+        kalman_filter : Any, optional
+            Kalman filter instance
+            
+        Returns
+        -------
+        np.ndarray or None
+            Updated last factor state (m,), or None if encoder unavailable
+        """
+        if not hasattr(self, 'encoder') or self.encoder is None:
+            _logger.warning(
+                f"{self.__class__.__name__} update(): Encoder not available, "
+                "cannot update factor state"
+            )
+            return None
+        
+        try:
+            # Extract factors using encoder
+            device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
+            self.encoder.eval()
+            X_tensor = torch.tensor(X_recent_std, device=device, dtype=torch.float32)
+            with torch.no_grad():
+                factors_raw = self.encoder(X_tensor).cpu().numpy()  # (T x m)
+            
+            # Validate factors shape
+            if factors_raw.ndim != 2 or factors_raw.shape[0] == 0:
+                _logger.warning(
+                    f"{self.__class__.__name__} update(): Invalid factors shape {factors_raw.shape}, "
+                    "cannot update factor state"
                 )
-                raise RuntimeError(error_msg)
-        
-        # Get processed data
-        X_torch = self._data_module.get_processed_data()
-        
-        # Get standardization parameters
-        Mx, Wx = self._data_module.get_std_params()
-        
-        return X_torch, Mx, Wx
+                return None
+            
+            # For DDFM: C = I (identity), R = small noise
+            m = factors_raw.shape[1]
+            C = np.eye(m)
+            R = np.eye(m) * 1e-8
+            
+            # Convert to torch: (m x T) format for Kalman filter
+            Y = torch.tensor(factors_raw.T, dtype=torch.float32)
+            params = self._result_to_torch_params(result, C=C, R=R)
+            
+            # Run Kalman smoother
+            kf = self._get_kalman_filter(kalman_filter)
+            kf_result = kf(Y, params['A'], params['C'], params['Q'],
+                          params['R'], params['Z_0'], params['V_0'])
+            
+            # Handle return value - Kalman filter returns (zsmooth, Vsmooth, VVsmooth, loglik)
+            if isinstance(kf_result, tuple) and len(kf_result) >= 1:
+                zsmooth = kf_result[0]
+            else:
+                zsmooth = kf_result
+            
+            # zsmooth is (m x (T+1)), transpose to ((T+1) x m), then get last row
+            if not isinstance(zsmooth, torch.Tensor):
+                zsmooth = torch.tensor(zsmooth, dtype=torch.float32)
+            
+            zsmooth_T = zsmooth.T  # ((T+1) x m)
+            if zsmooth_T.ndim == 2 and zsmooth_T.shape[0] > 0:
+                return zsmooth_T[-1, :].cpu().numpy()
+            else:
+                _logger.warning(
+                    f"{self.__class__.__name__} update(): Invalid zsmooth shape {zsmooth_T.shape}, "
+                    "cannot extract last state"
+                )
+                return None
+                
+        except Exception as e:
+            _logger.warning(
+                f"{self.__class__.__name__} update(): Failed to update factor state: {e}"
+            )
+            return None
     
     @property
     def result(self) -> BaseResult:
@@ -366,6 +553,7 @@ class BaseFactorModel(pl.LightningModule):
         self,
         horizon: Optional[int] = None,
         *,
+        history: Optional[int] = None,
         return_series: bool = True,
         return_factors: bool = True
     ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
@@ -375,6 +563,11 @@ class BaseFactorModel(pl.LightningModule):
         ----------
         horizon : int, optional
             Number of periods ahead to forecast. If None, uses default based on clock frequency.
+        history : int, optional
+            Number of historical periods to use for Kalman filter update before prediction.
+            If None, uses full history (default). If specified (e.g., 60), uses only the most
+            recent N periods for efficiency. Initial state (Z_0, V_0) is always estimated from
+            full history (including any new data beyond training period).
         return_series : bool, default True
             Whether to return forecasted series.
         return_factors : bool, default True
@@ -387,9 +580,89 @@ class BaseFactorModel(pl.LightningModule):
             - If both return_series and return_factors are True: (X_forecast, Z_forecast)
             - If only return_series is True: X_forecast
             - If only return_factors is True: Z_forecast
+            
+        Notes
+        -----
+        When history is specified, the method uses only the most recent N periods for
+        Kalman filter update, improving computational efficiency. The initial state
+        (Z_0, V_0) is always estimated from full history (including any new data beyond
+        training period), ensuring accuracy while maintaining efficiency.
         """
         pass
     
+    def update(
+        self,
+        X_std: np.ndarray,
+        *,
+        kalman_filter: Optional[Any] = None
+    ) -> 'BaseFactorModel':
+        """Update factor state with standardized data.
+        
+        This method permanently updates the last factor state (result.Z[-1, :])
+        using the provided standardized data. Users should handle all preprocessing
+        (masking, imputation, standardization) before calling this method.
+        
+        Parameters
+        ----------
+        X_std : np.ndarray
+            Standardized data array (T x N), where T is number of time periods
+            and N is number of series. Data should already be standardized using
+            result.Mx and result.Wx.
+        kalman_filter : Any, optional
+            Kalman filter instance. If None, uses default or model's kalman filter.
+            
+        Returns
+        -------
+        BaseFactorModel
+            Self for method chaining
+            
+        Examples
+        --------
+        >>> # Update state with new data, then predict
+        >>> model.update(X_std).predict(horizon=1)
+        >>> # Or update and predict separately
+        >>> model.update(X_std)
+        >>> forecast = model.predict(horizon=6)
+        """
+        self._check_trained()
+        
+        result = self.result  # Use property which ensures non-None after _check_trained()
+        
+        # Validate input shape
+        if not isinstance(X_std, np.ndarray):
+            X_std = np.asarray(X_std)
+        if X_std.ndim != 2:
+            raise ValueError(
+                f"{self.__class__.__name__} update(): X_std must be 2D array (T x N), "
+                f"got shape {X_std.shape}"
+            )
+        
+        # Handle NaN/Inf values
+        X_std = np.where(np.isfinite(X_std), X_std, np.nan)
+        
+        # Update factor state (model-specific)
+        if hasattr(self, 'encoder') and self.encoder is not None:
+            # DDFM: Extract factors via encoder, then use Kalman filter
+            Z_last_updated = self._update_factor_state_ddfm(
+                X_std, result, kalman_filter
+            )
+        else:
+            # DFM: Use Kalman filter directly on standardized data
+            Z_last_updated = self._update_factor_state_dfm(
+                X_std, result, kalman_filter or getattr(self, 'kalman', None)
+            )
+        
+        # Update result.Z[-1, :] permanently
+        if Z_last_updated is not None:
+            result.Z[-1, :] = Z_last_updated
+        else:
+            _logger.warning(
+                f"{self.__class__.__name__} update(): Failed to update factor state, "
+                f"keeping current state"
+            )
+        
+        return self
+    
     def get_result(self) -> BaseResult:
         """Extract result from trained model.
         
@@ -403,6 +676,7 @@ class BaseFactorModel(pl.LightningModule):
         """
         raise NotImplementedError("Subclasses must implement get_result()")
     
+    
     def _load_config_common(
         self,
         source: Optional[Union[str, Path, Dict[str, Any], DFMConfig, ConfigSource]] = None,
